@@ -16,8 +16,14 @@
 #include <unistd.h>
 #include <errno.h>
 
+#ifdef _WIN32
+#include "win/fmemopen.h"
+#include "win/basename_patch.h"
+#endif
+
 #include "misc/io.h"
 #include "misc/misc.h"
+#include "misc/opts.h"
 #include "misc/version.h"
 #include "misc/debug.h"
 #include "misc/cppmap.h"
@@ -48,6 +54,8 @@ static void bart_exit_cleanup(void)
 
 	io_memory_cleanup();
 
+	opt_free_strdup();
+
 #ifdef FFTWTHREADS
 	MANGLE(fftwf_cleanup_threads)();
 #endif
@@ -59,7 +67,7 @@ static void bart_exit_cleanup(void)
 
 
 struct {
-	
+
 	int (*main_fun)(int argc, char* argv[]);
 	const char* name;
 
@@ -90,7 +98,7 @@ int main_bart(int argc, char* argv[argc])
 {
 	char* bn = basename(argv[0]);
 
-	if (0 == strcmp(bn, "bart")) {
+	if (0 == strcmp(bn, "bart") || 0 == strcmp(bn, "bart.exe")) {
 
 		if (1 == argc) {
 
@@ -106,18 +114,23 @@ int main_bart(int argc, char* argv[argc])
 			"/usr/lib/bart/commands/",
 		};
 
-		for (unsigned int i = 0; i < ARRAY_SIZE(tpath); i++) {
+		for (int i = 0; i < (int)ARRAY_SIZE(tpath); i++) {
 
 			if (NULL == tpath[i])
 				continue;
 
 			size_t len = strlen(tpath[i]) + strlen(argv[1]) + 2;
 
-			char cmd[len];
-			size_t r = snprintf(cmd, len, "%s/%s", tpath[i], argv[1]);
-			assert(r < len);
+			char (*cmd)[len] = xmalloc(sizeof *cmd);
+			size_t r = snprintf(*cmd, len, "%s/%s", tpath[i], argv[1]);
 
-			if (-1 == execv(cmd, argv + 1)) {
+			if (r >= len) {
+
+				perror("Commandline too long");
+				return 1;
+			}
+
+			if (-1 == execv(*cmd, argv + 1)) {
 
 				// only if it doesn't exist - try builtin
 
@@ -131,6 +144,8 @@ int main_bart(int argc, char* argv[argc])
 
 				assert(0);
 			}
+
+			xfree(cmd);
 		}
 
 		return main_bart(argc - 1, argv + 1);
@@ -171,6 +186,11 @@ int bart_command(int len, char* buf, int argc, char* argv[])
 
 	if (NULL != bart_output) {
 
+#ifdef _WIN32
+		rewind(bart_output);
+		fread(buf, 1, len, bart_output);
+#endif
+
 		fclose(bart_output);	// write final nul
 		bart_output = NULL;
 	}
