@@ -1,10 +1,10 @@
 /* Copyright 2013-2015. The Regents of the University of California.
- * Copyright 2015-2017. Martin Uecker.
+ * Copyright 2015-2021. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2012-2021 Martin Uecker <uecker@tugraz.at>
  * 2013 Dara Bahri <dbahri123@gmail.com>
  * 2015 Siddharth Iyer <sid8795@gmail.com>
  */
@@ -33,7 +33,6 @@
 
 
 
-static const char usage_str[] = "<kspace> <sensitivites> [<ev-maps>]";
 static const char help_str[] =
 		"Estimate coil sensitivities using ESPIRiT calibration.\n"
 		"Optionally outputs the eigenvalue maps.";
@@ -44,7 +43,18 @@ static const char help_str[] =
 
 int main_ecalib(int argc, char* argv[argc])
 {
-	long calsize[3] = { 24, 24, 24 }; 
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+	const char* emaps_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &in_file, "kspace"),
+		ARG_OUTFILE(true, &out_file, "sensitivities"),
+		ARG_OUTFILE(false, &emaps_file, "ev-maps"),
+	};
+
+	long calsize[3] = { 24, 24, 24 };
 	int maps = 2;
 	bool one = false;
 	bool calcen = false;
@@ -67,6 +77,7 @@ int main_ecalib(int argc, char* argv[argc])
 		OPT_SET('1', &one, "perform only first part of the calibration"),
 		OPT_CLEAR('P', &conf.rotphase, "Do not rotate the phase with respect to the first principal component"),
 		OPT_CLEAR('O', &conf.orthiter, "()"),
+		OPTL_INT('i', "orthiter", &conf.num_orthiter, "orthiter", "()"),
 		OPT_FLOAT('b', &conf.perturb, "", "()"),
 		OPT_SET('V', &print_svals, "()"),
 		OPT_SET('C', &calcen, "()"),
@@ -78,7 +89,7 @@ int main_ecalib(int argc, char* argv[argc])
 		OPT_INT('d', &debug_level, "level", "Debug level"),
 	};
 
-	cmdline(&argc, argv, 2, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	if (-1. != conf.percentsv)
 		conf.threshold = -1.;
@@ -103,7 +114,7 @@ int main_ecalib(int argc, char* argv[argc])
 	int N = DIMS;
 	long ksp_dims[N];
 
-	complex float* in_data = load_cfl(argv[1], N, ksp_dims);
+	complex float* in_data = load_cfl(in_file, N, ksp_dims);
 
 	
 	// assert((kdims[0] < calsize_ro) && (kdims[1] < calsize_ro) && (kdims[2] < calsize_ro));
@@ -115,14 +126,13 @@ int main_ecalib(int argc, char* argv[argc])
 	long cal_dims[N];
 	complex float* cal_data = NULL;
 
-	 if (!calcen) {
+	if (!calcen) {
 
 #ifdef USE_CC_EXTRACT_CALIB
 		cal_data = cc_extract_calib(cal_dims, calsize, ksp_dims, in_data);
 #else
 		cal_data = extract_calib(cal_dims, calsize, ksp_dims, in_data, false);
 #endif
-
 	} else {
 	
 		for (int i = 0; i < 3; i++)
@@ -132,6 +142,7 @@ int main_ecalib(int argc, char* argv[argc])
 			cal_dims[i] = ksp_dims[i];
 
 		cal_data = md_alloc(5, cal_dims, CFL_SIZE);
+
 		md_resize_center(5, cal_dims, cal_data, ksp_dims, in_data, CFL_SIZE);
 	 }
 
@@ -143,11 +154,11 @@ int main_ecalib(int argc, char* argv[argc])
 
 
 	 long channels = cal_dims[3];
-	 unsigned int K = conf.kdims[0] * conf.kdims[1] * conf.kdims[2] * channels;
+	 int K = conf.kdims[0] * conf.kdims[1] * conf.kdims[2] * channels;
+
 	 float svals[K];
 
-
-	 for (unsigned int i = 0; i < 3; i++)
+	 for (int i = 0; i < 3; i++)
 		if ((1 == cal_dims[i]) && (1 != ksp_dims[i]))
 			error("Calibration region not found!\n");
 
@@ -160,8 +171,12 @@ int main_ecalib(int argc, char* argv[argc])
 
 	(conf.usegpu ? num_init_gpu : num_init)();
 
-        if ((conf.var < 0.) && (conf.weighting || (conf.crop < 0.)))
-		conf.var = estvar_calreg(conf.kdims, cal_dims, cal_data);
+        if ((conf.var < 0.) && (conf.weighting || (conf.crop < 0.))) {
+
+		const char* toolbox = getenv("TOOLBOX_PATH");
+
+		conf.var = estvar_calreg(toolbox, conf.kdims, cal_dims, cal_data);
+	}
 
 	if (one) {
 
@@ -179,7 +194,7 @@ int main_ecalib(int argc, char* argv[argc])
 
 		calone(&conf, cov_dims, imgcov, K, svals, cal_dims, cal_data);
 
-		complex float* out = create_cfl(argv[2], 4, cov_dims);
+		complex float* out = create_cfl(out_file, 4, cov_dims);
 		md_copy(4, cov_dims, out, imgcov, CFL_SIZE);
 		unmap_cfl(4, cov_dims, out);
 
@@ -213,12 +228,7 @@ int main_ecalib(int argc, char* argv[argc])
 		map_dims[COIL_DIM] = 1;
 		map_dims[MAPS_DIM] = maps;
 
-		const char* emaps_file = NULL;
-
-		if (4 == argc)
-			emaps_file = argv[3];
-
-		complex float* out_data = create_cfl(argv[2], N, out_dims);
+		complex float* out_data = create_cfl(out_file, N, out_dims);
 		complex float* emaps = (emaps_file ? create_cfl : anon_cfl)(emaps_file, N, map_dims);
 
 		calib(&conf, out_dims, out_data, emaps, K, svals, cal_dims, cal_data); 
@@ -228,15 +238,14 @@ int main_ecalib(int argc, char* argv[argc])
 	}
 
 
-	if (print_svals) {
-
-		for (unsigned int i = 0; i < K; i++)
+	if (print_svals)
+		for (int i = 0; i < K; i++)
 			printf("SVALS %d %f\n", i, svals[i]);
-	}
 
 	printf("Done.\n");
 
 	unmap_cfl(N, ksp_dims, in_data);
+
 	md_free(cal_data);
 
 	return 0;
