@@ -3,7 +3,8 @@
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2015 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2015-2017 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2020 Sebastian Rosenzweig <sebastian.rosenzweig@med.uni-goettingen.de>
  */
 
 #include <stdbool.h>
@@ -30,29 +31,56 @@
 #endif
 
 
-static const char usage_str[] = "<input> <output>";
-static const char help_str[] = "Apply filter.\n";
+static const char help_str[] = "Apply filter.";
 
 
-int main_filter(int argc, char* argv[])
+int main_filter(int argc, char* argv[argc])
 {
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &in_file, "input"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
 	int len = -1;
 	int dim = -1;
+	int med = -1;
+	int mavg = -1;
+	bool geom = false;
 
 	const struct opt_s opts[] = {
 
-		OPT_INT('m', &dim, "dim", "median filter along dimension dim"),
+		OPT_INT('m', &med, "dim", "median filter along dimension dim"),
 		OPT_INT('l', &len, "len", "length of filter"),
+		OPT_SET('G', &geom, "geometric median"),
+		OPT_INT('a', &mavg, "dim", "Moving average filter along dimension dim"),
 	};
 
-	cmdline(&argc, argv, 2, 2, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
+	char filter_type;
+
+	assert (med == -1 || mavg == -1);
+
+	if (med >= 0) {
+
+		dim = med;
+		filter_type = 'm';
+
+	} else if (mavg >= 0) {
+
+		dim = mavg;
+		filter_type = 'a';
+	}
 
 	long in_dims[DIMS];
-	
-	complex float* in_data = load_cfl(argv[1], DIMS, in_dims);
+
+	complex float* in_data = load_cfl(in_file, DIMS, in_dims);
 
 	assert(dim >= 0);
 	assert(dim < DIMS);
@@ -77,12 +105,24 @@ int main_filter(int argc, char* argv[])
 	long out_dims[DIMS];
 	md_copy_dims(DIMS, out_dims, tmp_dims);
 
-	complex float* out_data = create_cfl(argv[2], DIMS, out_dims);
+	complex float* out_data = create_cfl(out_file, DIMS, out_dims);
 
-	md_medianz2(DIMS + 1, DIMS, tmp_dims, tmp2_strs, out_data, tmp_strs, in_data);
+	switch (filter_type) {
+
+	case 'm':
+
+		(geom ? md_geometric_medianz2 : md_medianz2)(DIMS + 1, DIMS, tmp_dims, tmp2_strs, out_data, tmp_strs, in_data);
+		break;
+
+	case 'a':
+
+		md_moving_avgz2(DIMS + 1, DIMS, tmp_dims, tmp2_strs, out_data, tmp_strs, in_data);
+		break;
+	}
 
 	unmap_cfl(DIMS, in_dims, in_data);
 	unmap_cfl(DIMS, out_dims, out_data);
+
 	return 0;
 }
 
