@@ -9,6 +9,9 @@
 
 #include "misc/cppwrap.h"
 
+enum IN_TYPE { IN_UNDEFINED, IN_STATIC, IN_BATCH, IN_OPTIMIZE, IN_BATCH_GENERATOR, IN_BATCHNORM };
+enum OUT_TYPE { OUT_UNDEFINED, OUT_STATIC, OUT_OPTIMIZE, OUT_BATCHNORM };
+
 #ifndef NUM_INTERNAL
 // #warning "Use of private interfaces"
 #endif
@@ -17,15 +20,23 @@
 #include "misc/nested.h"
 
 struct vec_iter_s;
+struct iter_dump_s;
 
+#ifndef MD_IS_SET
+#define MD_BIT(x) (1ul << (x))
+#define MD_IS_SET(x, y)	((x) & MD_BIT(y))
+#define MD_CLEAR(x, y) ((x) & ~MD_BIT(y))
+#define MD_SET(x, y)	((x) | MD_BIT(y))
+#endif
 
 #ifndef ITER_OP_DATA_S
 #define ITER_OP_DATA_S
 typedef struct iter_op_data_s { TYPEID* TYPEID; } iter_op_data;
 #endif
 typedef void (*iter_op_fun_t)(iter_op_data* data, float* dst, const float* src);
-typedef void (*iter_nlop_fun_t)(iter_op_data* data, int N, float* args[N]);
+typedef void (*iter_nlop_fun_t)(iter_op_data* data, int N, float* args[N], unsigned long der_out, unsigned long der_in);
 typedef void (*iter_op_p_fun_t)(iter_op_data* data, float rho, float* dst, const float* src);
+typedef void (*iter_op_arr_fun_t)(iter_op_data* data, int NO, unsigned long oflags, float* dst[NO], int NI, unsigned long iflags, const float* src[NI]);
 
 struct iter_op_s {
 
@@ -45,6 +56,12 @@ struct iter_op_p_s {
 	iter_op_data* data;
 };
 
+struct iter_op_arr_s {
+
+	iter_op_arr_fun_t fun;
+	iter_op_data* data;
+};
+
 inline void iter_op_call(struct iter_op_s op, float* dst, const float* src)
 {
 	op.fun(op.data, dst, src);
@@ -52,7 +69,12 @@ inline void iter_op_call(struct iter_op_s op, float* dst, const float* src)
 
 inline void iter_nlop_call(struct iter_nlop_s op, int N, float* args[N])
 {
-	op.fun(op.data, N, args);
+	op.fun(op.data, N, args, ~0ul, ~0ul);
+}
+
+inline void iter_nlop_call_select_der(struct iter_nlop_s op, int N, float* args[N], unsigned long der_out, unsigned long der_in)
+{
+	op.fun(op.data, N, args, der_out, der_in);
 }
 
 inline void iter_op_p_call(struct iter_op_p_s op, float rho, float* dst, const float* src)
@@ -60,9 +82,14 @@ inline void iter_op_p_call(struct iter_op_p_s op, float rho, float* dst, const f
 	op.fun(op.data, rho, dst, src);
 }
 
+inline void iter_op_arr_call(struct iter_op_arr_s op, int NO, unsigned long oflags, float* dst[NO], int NI, unsigned long iflags, const float* src[NI])
+{
+	op.fun(op.data, NO, oflags, dst, NI, iflags, src);
+}
 
 
 struct iter_monitor_s;
+struct monitor_iter6_s;
 
 float conjgrad(unsigned int maxiter, float l2lambda, float epsilon,
 	long N,
@@ -88,6 +115,21 @@ void landweber_sym(unsigned int maxiter, float epsilon, float alpha,
 	float* x, const float* b,
 	struct iter_monitor_s* monitor);
 
+void sgd(unsigned int epochs, unsigned int batches,
+	float learning_rate, float batchnorm_momentum,
+	float learning_rate_schedule[epochs][batches],
+	long NI, long isize[NI], enum IN_TYPE in_type[NI], float* x[NI],
+	long NO, long osize[NO], enum OUT_TYPE out_type[NI],
+	int N_batch, int N_total,
+	const struct vec_iter_s* vops,
+	struct iter_nlop_s nlop,
+	struct iter_op_arr_s adj,
+	struct iter_op_p_s update[NI],
+	struct iter_op_p_s prox[NI],
+	struct iter_nlop_s nlop_batch_gen,
+	struct iter_op_s callback,
+	struct monitor_iter6_s* monitor,
+	const struct iter_dump_s* dump);
 
 /**
  * Store information about iterative algorithm.
@@ -183,7 +225,19 @@ void chambolle_pock(unsigned int maxiter, float epsilon, float tau, float sigma,
 	float* x,
 	struct iter_monitor_s* monitor);
 
+void iPALM(	long NI, long isize[__VLA(NI)], enum IN_TYPE in_type[__VLA(NI)], float* x[__VLA(NI)], float* x_old[__VLA(NI)],
+		long NO, long osize[__VLA(NO)], enum OUT_TYPE out_type[__VLA(NO)],
+		int numbatches, int epoch_start, int epoch_end,
+		const struct vec_iter_s* vops,
+		float alpha[__VLA(NI)], float beta[__VLA(NI)], _Bool convex[__VLA(NI)], _Bool trivial_stepsize, _Bool reduce_momentum,
+		float L[__VLA(NI)], float Lmin, float Lmax, float Lshrink, float Lincrease,
+		struct iter_nlop_s nlop,
+		struct iter_op_arr_s adj,
+		struct iter_op_p_s prox[__VLA(NI)],
+		float batchnorm_momentum,
+		struct iter_nlop_s nlop_batch_gen,
+		struct iter_op_s callback, struct monitor_iter6_s* monitor, const struct iter_dump_s* dump);
+
 #include "misc/cppwrap.h"
 
 #endif // __ITALGOS_H
-
