@@ -1,5 +1,5 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2016-2019. Martin Uecker.
+ * Copyright 2016-2021. Uecker Lab. University Medical Center Göttingen.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
@@ -10,18 +10,22 @@
 
 #include <complex.h>
 #include <stdbool.h>
+#include <stdio.h>
 #include <assert.h>
 
+#include "misc/types.h"
 #include "num/multind.h"
 #include "num/flpmath.h"
 #include "num/iovec.h"
 #include "num/ops_p.h"
 #include "num/ops.h"
+#include "num/ops_graph.h"
 
 #include "misc/misc.h"
 #include "misc/shrdptr.h"
 #include "misc/debug.h"
 #include "misc/shrdptr.h"
+#include "misc/list.h"
 
 #include "linop.h"
 
@@ -36,6 +40,9 @@ struct shared_data_s {
 	linop_data_t* data;
 	del_fun_t del;
 
+	enum LINOP_TYPE lop_type;
+	lop_graph_t get_graph;
+
 	struct shared_ptr_s sptr;
 
 	union {
@@ -85,26 +92,93 @@ static void sptr_del(const struct shared_ptr_s* p)
 	data->del(data->data);
 }
 
+const char* lop_type_str[] = {
+
+	[LOP_FORWARD] = "forward",
+	[LOP_ADJOINT] = "adjoint",
+	[LOP_NORMAL] = "normal",
+	[LOP_NORMAL_INV] = "normal inversion",
+};
+
+static const struct graph_s* lop_get_graph_default(const struct operator_s* op, linop_data_t* data, enum LINOP_TYPE lop_type)
+{
+	const char* name = ptr_printf("linop\\n%s\\n%s", data->TYPEID->name, lop_type_str[lop_type]);
+
+	auto result = create_graph_operator(op, name);
+
+	xfree(name);
+
+	return result;
+}
+
+static const struct graph_s* operator_linop_get_graph(const struct operator_s* op)
+{
+	auto data = CAST_DOWN(shared_data_s, operator_get_data(op));
+
+	if (NULL != data->get_graph)
+		return data->get_graph(op, data->data, data->lop_type);
+	else
+		return lop_get_graph_default(op, data->data, data->lop_type);
+}
+
+struct linop_s* linop_from_ops(
+	const struct operator_s* forward,
+	const struct operator_s* adjoint,
+	const struct operator_s* normal,
+	const struct operator_p_s* norm_inv)
+{
+	assert(NULL != forward);
+	assert(NULL != adjoint);
+
+	auto dom = operator_domain(forward);
+	auto cod = operator_codomain(forward);
+
+	assert(iovec_check(operator_codomain(adjoint), dom->N, dom->dims, dom->strs));
+	assert(iovec_check(operator_domain(adjoint), cod->N, cod->dims, cod->strs));
+
+	PTR_ALLOC(struct linop_s, x);
+	x->forward = operator_ref(forward);
+	x->adjoint = operator_ref(adjoint);
+
+	if (NULL != normal) {
+
+		assert(iovec_check(operator_codomain(normal), dom->N, dom->dims, dom->strs));
+		assert(iovec_check(operator_domain(normal), dom->N, dom->dims, dom->strs));
+
+		x->normal = operator_ref(normal);
+	} else {
+
+		x->normal = operator_chain(forward, adjoint);
+	}
+
+	if (NULL != norm_inv)
+		x->norm_inv = operator_p_ref(norm_inv);
+	else
+		x->norm_inv = NULL;
+
+	return PTR_PASS(x);
+}
 
 /**
  * Create a linear operator (with strides)
  */
-struct linop_s* linop_create2(unsigned int ON, const long odims[ON], const long ostrs[ON],
+struct linop_s* linop_with_graph_create2(unsigned int ON, const long odims[ON], const long ostrs[ON],
 				unsigned int IN, const long idims[IN], const long istrs[IN],
 				linop_data_t* data, lop_fun_t forward, lop_fun_t adjoint, lop_fun_t normal,
-				lop_p_fun_t norm_inv, del_fun_t del)
+				lop_p_fun_t norm_inv, del_fun_t del,
+				lop_graph_t get_graph)
 {
 	PTR_ALLOC(struct linop_s, lo);
 
 	struct shared_data_s* shared_data[4];
 
-	for (unsigned int i = 0; i < 4; i++) {
+	for (int i = 0; i < 4; i++) {
 
 		shared_data[i] = TYPE_ALLOC(struct shared_data_s);
 		SET_TYPEID(shared_data_s, shared_data[i]);
 	}
 
-	for (unsigned int i = 0; i < 4; i++) {
+	for (int i = 0; i < 4; i++) {
 
 		shared_data[i]->data = data;
 		shared_data[i]->del = del;
@@ -113,6 +187,8 @@ struct linop_s* linop_create2(unsigned int ON, const long odims[ON], const long
 			shared_ptr_init(&shared_data[i]->sptr, sptr_del);
 		else
 			shared_ptr_copy(&shared_data[i]->sptr, &shared_data[0]->sptr);
+
+		shared_data[i]->get_graph = get_graph;
 	}
 
 	shared_data[0]->u.apply = forward;
@@ -120,15 +196,27 @@ struct linop_s* linop_create2(unsigned int ON, const long odims[ON], const long
 	shared_data[2]->u.apply = normal;
 	shared_data[3]->u.apply_p = norm_inv;
 
+	shared_data[0]->lop_type = LOP_FORWARD;
+	shared_data[1]->lop_type = LOP_ADJOINT;
+	shared_data[2]->lop_type = LOP_NORMAL;
+	shared_data[3]->lop_type = LOP_NORMAL_INV;
+
 	assert((NULL != forward));
 	assert((NULL != adjoint));
 
-	lo->forward = operator_create2(ON, odims, ostrs, IN, idims, istrs, CAST_UP(shared_data[0]), shared_apply, shared_del);
-	lo->adjoint = operator_create2(IN, idims, istrs, ON, odims, ostrs, CAST_UP(shared_data[1]), shared_apply, shared_del);
+	lo->forward = operator_generic_create2(	2, (bool[2]){ true, false },
+						(unsigned int[2]){ ON, IN }, (const long* [2]){ odims, idims }, (const long* [2]){ ostrs, istrs },
+						CAST_UP(shared_data[0]), shared_apply, shared_del, operator_linop_get_graph);
+
+	lo->adjoint = operator_generic_create2(	2, (bool[2]){ true, false },
+						(unsigned int[2]){ IN, ON }, (const long* [2]){ idims, odims }, (const long* [2]){ istrs, ostrs },
+						CAST_UP(shared_data[1]), shared_apply, shared_del, operator_linop_get_graph);
 
 	if (NULL != normal) {
 
-		lo->normal = operator_create2(IN, idims, istrs, IN, idims, istrs, CAST_UP(shared_data[2]), shared_apply, shared_del);
+		lo->normal = operator_generic_create2(	2, (bool[2]){ true, false },
+							(unsigned int[2]){ IN, IN }, (const long* [2]){ idims, idims }, (const long* [2]){ istrs, istrs },
+							CAST_UP(shared_data[2]), shared_apply, shared_del, operator_linop_get_graph);
 
 	} else {
 
@@ -155,6 +243,44 @@ struct linop_s* linop_create2(unsigned int ON, const long odims[ON], const long
 	return PTR_PASS(lo);
 }
 
+/**
+ * Create a linear operator (with strides)
+ */
+struct linop_s* linop_create2(unsigned int ON, const long odims[ON], const long ostrs[ON],
+				unsigned int IN, const long idims[IN], const long istrs[IN],
+				linop_data_t* data, lop_fun_t forward, lop_fun_t adjoint, lop_fun_t normal,
+				lop_p_fun_t norm_inv, del_fun_t del)
+{
+	return linop_with_graph_create2(ON, odims, ostrs, IN, idims, istrs, data, forward, adjoint, normal, norm_inv, del, NULL);
+}
+
+
+/**
+ * Create a linear operator (without strides)
+ *
+ * @param N number of dimensions
+ * @param odims dimensions of output (codomain)
+ * @param idims dimensions of input (domain)
+ * @param data data for applying the operator
+ * @param forward function for applying the forward operation, A
+ * @param adjoint function for applying the adjoint operation, A^H
+ * @param normal function for applying the normal equations operation, A^H A
+ * @param norm_inv function for applying the pseudo-inverse operation, (A^H A + mu I)^-1
+ * @param del function for freeing the data
+ * @param
+ * @param
+ */
+struct linop_s* linop_with_graph_create(unsigned int ON, const long odims[ON], unsigned int IN, const long idims[IN], linop_data_t* data,
+					lop_fun_t forward, lop_fun_t adjoint, lop_fun_t normal, lop_p_fun_t norm_inv, del_fun_t del,
+					lop_graph_t get_graph)
+{
+	long ostrs[ON];
+	long istrs[IN];
+	md_calc_strides(ON, ostrs, odims, CFL_SIZE);
+	md_calc_strides(IN, istrs, idims, CFL_SIZE);
+
+	return linop_with_graph_create2(ON, odims, ostrs, IN, idims, istrs, data, forward, adjoint, normal, norm_inv, del, get_graph);
+}
 
 /**
  * Create a linear operator (without strides)
@@ -188,10 +314,27 @@ struct linop_s* linop_create(unsigned int ON, const long odims[ON], unsigned int
 const linop_data_t* linop_get_data(const struct linop_s* ptr)
 {
 	auto sdata = CAST_MAYBE(shared_data_s, operator_get_data(ptr->forward));
+
+	return (sdata == NULL) ? NULL : sdata->data;
+}
+
+/**
+ * Return the data associated with the linear operator and look into reshape containers
+ *
+ * @param ptr linear operator
+ */
+const linop_data_t* linop_get_data_nested(const struct linop_s* ptr)
+{
+	const struct operator_s* op = ptr->forward;
+	while(NULL != get_in_reshape(op))
+		op = get_in_reshape(op);
+
+	auto sdata = CAST_MAYBE(shared_data_s, operator_get_data(op));
 	return sdata == NULL ? NULL : sdata->data;
 }
 
 
+
 /**
  * Make a copy of a linear operator
  * @param x linear operator
@@ -208,6 +351,38 @@ extern const struct linop_s* linop_clone(const struct linop_s* x)
 	return PTR_PASS(lo);
 }
 
+/**
+ * Return the adjoint linop
+ * @param x linear operator
+ */
+extern const struct linop_s* linop_get_adjoint(const struct linop_s* x)
+{
+	PTR_ALLOC(struct linop_s, lo);
+
+	lo->forward = operator_ref(x->adjoint);
+	lo->adjoint = operator_ref(x->forward);
+	lo->normal = operator_chain(x->adjoint, x->forward);
+	lo->norm_inv = NULL;
+
+	return PTR_PASS(lo);
+}
+
+/**
+ * Return the normal linop
+ * @param x linear operator
+ */
+extern const struct linop_s* linop_get_normal(const struct linop_s* x)
+{
+	PTR_ALLOC(struct linop_s, lo);
+
+	lo->forward = operator_ref(x->normal);
+	lo->adjoint = operator_ref(x->normal);
+	lo->normal = operator_chain(x->normal, x->normal);
+	lo->norm_inv = NULL;
+
+	return PTR_PASS(lo);
+}
+
 
 /**
  * Apply the forward operation of a linear operator: y = A x
@@ -221,7 +396,7 @@ extern const struct linop_s* linop_clone(const struct linop_s* x)
  * @param sdims dimensions of the input (domain)
  * @param src input data
  */
-void linop_forward(const struct linop_s* op, unsigned int DN, const long ddims[DN], complex float* dst, 
+void linop_forward(const struct linop_s* op, unsigned int DN, const long ddims[DN], complex float* dst,
 			unsigned int SN, const long sdims[SN], const complex float* src)
 {
 	assert(op->forward);
@@ -267,10 +442,12 @@ void linop_pseudo_inv(const struct linop_s* op, float lambda,
 			unsigned int SN, const long sdims[SN], const complex float* src)
 {
 	complex float* adj = md_alloc_sameplace(DN, ddims, CFL_SIZE, dst);
+
 	linop_adjoint(op, DN, ddims, adj, SN, sdims, src);
 
 	assert(op->norm_inv);
 	operator_p_apply(op->norm_inv, lambda, DN, ddims, dst, DN, ddims, adj);
+
 	md_free(adj);
 }
 
@@ -399,6 +576,11 @@ struct linop_s* linop_null_create2(unsigned int NO, const long odims[NO], const
 	return PTR_PASS(c);
 }
 
+bool linop_is_null(const struct linop_s* lop)
+{
+	return operator_zero_or_null_p(lop->forward);
+}
+
 
 
 struct linop_s* linop_null_create(unsigned int NO, const long odims[NO], unsigned int NI, const long idims[NI])
@@ -459,14 +641,26 @@ struct linop_s* linop_chain_FF(const struct linop_s* a, const struct linop_s* b)
 }
 
 
-struct linop_s* linop_chainN(unsigned int N, struct linop_s* a[N])
+struct linop_s* linop_chainN(int N, struct linop_s* a[N])
 {
 	assert(N > 0);
 
-	if (1 == N)
-		return a[0];
+	struct linop_s* result = (struct linop_s*)linop_clone(a[0]);
+	for (int i = 1; i < N; i++)
+		result = linop_chain_FF(result, linop_clone(a[i]));
 
-	return linop_chain(a[0], linop_chainN(N - 1, a + 1));	// FIXME: free intermed.
+	return result;
+}
+
+struct linop_s* linop_chainN_F(int N, struct linop_s* a[N])
+{
+	assert(N > 0);
+
+	struct linop_s* result = a[0];
+	for (int i = 1; i < N; i++)
+		result = linop_chain_FF(result, a[i]);
+
+	return result;
 }
 
 
@@ -490,13 +684,23 @@ struct linop_s* linop_stack(int D, int E, const struct linop_s* a, const struct
 	if (NULL == bn)
 		bn = operator_chain(b->forward, b->adjoint);
 
-	c->normal = operator_stack(D, D, an, bn);
+	c->normal = operator_stack(E, E, an, bn);
 
 	c->norm_inv = NULL;
 
 	return PTR_PASS(c);
 }
 
+struct linop_s* linop_stack_FF(int D, int E, const struct linop_s* a, const struct linop_s* b)
+{
+	auto result = linop_stack(D, E, a, b);
+
+	linop_free(a);
+	linop_free(b);
+
+	return result;
+}
+
 
 
 
@@ -515,11 +719,13 @@ struct linop_s* linop_loop(unsigned int D, const long dims[D], struct linop_s* o
 	return PTR_PASS(op2);
 }
 
-
-struct linop_s* linop_copy_wrapper(unsigned int D, const long istrs[D], const long ostrs[D],  struct linop_s* op)
+struct linop_s* linop_copy_wrapper2(int DI, const long istrs[DI], int DO, const long ostrs[DO],  struct linop_s* op)
 {
 	PTR_ALLOC(struct linop_s, op2);
 
+	assert((int)(linop_codomain(op)->N) == DO);
+	assert((int)(linop_domain(op)->N) == DI);
+
 	const long* strsx[2] = { ostrs, istrs };
 	const long* strsy[2] = { istrs, ostrs };
 	const long* strsz[2] = { istrs, istrs };
@@ -532,6 +738,22 @@ struct linop_s* linop_copy_wrapper(unsigned int D, const long istrs[D], const lo
 	return PTR_PASS(op2);
 }
 
+struct linop_s* linop_copy_wrapper(unsigned int D, const long istrs[D], const long ostrs[D],  struct linop_s* op)
+{
+	return linop_copy_wrapper2(D, istrs, D, ostrs, op);
+}
+
+struct linop_s* linop_gpu_wrapper(struct linop_s* op)
+{
+	PTR_ALLOC(struct linop_s, op2);
+
+	op2->forward = operator_gpu_wrapper(op->forward);
+	op2->adjoint = operator_gpu_wrapper(op->adjoint);
+	op2->normal = (NULL == op->normal) ? NULL : operator_gpu_wrapper(op->normal);
+	op2->norm_inv = NULL; // FIXME
+
+	return PTR_PASS(op2);
+}
 
 
 
@@ -545,10 +767,12 @@ void linop_free(const struct linop_s* op)
 {
 	if (NULL == op)
 		return;
+
 	operator_free(op->forward);
 	operator_free(op->adjoint);
 	operator_free(op->normal);
 	operator_p_free(op->norm_inv);
+
 	xfree(op);
 }
 
@@ -569,32 +793,18 @@ struct linop_s* linop_plus(const struct linop_s* a, const struct linop_s* b)
 
 	c->forward = operator_plus_create(a->forward, b->forward);
 	c->adjoint = operator_plus_create(a->adjoint, b->adjoint);
-
-	if ((NULL != a->normal) && (NULL != b->normal))
-		c->normal = operator_plus_create(a->normal, b->normal);
-
-	if ((NULL != a->normal) && (NULL == b->normal)) {
-
-		auto tmp = operator_chain(b->forward, b->adjoint);
-		c->normal = operator_plus_create(a->normal, tmp);
-		operator_free(tmp);
-	}
-
-	if ((NULL == a->normal) && (NULL != b->normal)) {
-
-		auto tmp = operator_chain(a->forward, a->adjoint);
-		c->normal = operator_plus_create(tmp, b->normal);
-		operator_free(tmp);
-	}
-
-	if ((NULL == a->normal) && (NULL == b->normal))
-		c->normal = NULL;
-
+	c->normal = operator_chain(c->forward, c->adjoint);
 	c->norm_inv = NULL;
 
-	return PTR_PASS(c);
+	auto result = PTR_PASS(c);
+	auto result_optimized = graph_optimize_linop(result);
+
+	linop_free(result);
+
+	return result_optimized;
 }
 
+
 struct linop_s* linop_plus_FF(const struct linop_s* a, const struct linop_s* b)
 {
 	auto x = linop_plus(a, b);
@@ -606,27 +816,26 @@ struct linop_s* linop_plus_FF(const struct linop_s* a, const struct linop_s* b)
 }
 
 
-struct linop_s* linop_reshape_in(const struct linop_s* op, unsigned int NI, const long idims[NI]) {
-
+struct linop_s* linop_reshape_in(const struct linop_s* op, unsigned int NI, const long idims[NI])
+{
 	PTR_ALLOC(struct linop_s, c);
 
 	c->forward = operator_reshape(op->forward, 1, NI, idims);
 	c->adjoint = operator_reshape(op->adjoint, 0, NI, idims);
+	c->normal = NULL;
+	c->norm_inv = NULL;
 
 	if (NULL != op->normal) {
 
 		auto tmp = operator_reshape(op->normal, 1, NI, idims);
-		c->normal = operator_reshape(tmp, 0, NI, idims);
-		operator_free(tmp);
-	} else {
 
-		c->normal = NULL;
+		c->normal = operator_reshape(tmp, 0, NI, idims);
+
+		operator_free(tmp);
 	}
 
 	if (NULL != op->norm_inv)
 		c->norm_inv = operator_p_reshape_out_F(operator_p_reshape_in(op->norm_inv, NI, idims), NI, idims);
-	else
-		c->norm_inv = NULL;
 
 	return PTR_PASS(c);
 }
@@ -646,13 +855,128 @@ struct linop_s* linop_reshape_out(const struct linop_s* op, unsigned int NO, con
 struct linop_s* linop_reshape_in_F(const struct linop_s* op, unsigned int NI, const long idims[NI])
 {
 	auto result = linop_reshape_in(op, NI, idims);
+
 	linop_free(op);
+
 	return result;
 }
 
 struct linop_s* linop_reshape_out_F(const struct linop_s* op, unsigned int NO, const long odims[NO])
 {
 	auto result = linop_reshape_out(op, NO, odims);
+
 	linop_free(op);
+
 	return result;
 }
+
+const linop_data_t* operator_get_linop_data(const struct operator_s* op)
+{
+	auto data = CAST_MAYBE(shared_data_s, operator_get_data(op));
+
+	if (NULL == data)
+		return NULL;
+	else
+		return data->data;
+}
+
+
+static enum node_identic node_identify_linop(const struct node_s* _a, const struct node_s* _b)
+{
+	auto a = get_operator_from_node(_a);
+	auto b = get_operator_from_node(_b);
+
+	if ((NULL == a) || (a != b) || (NULL == operator_get_linop_data(a)))
+		return NODE_NOT_IDENTICAL;
+
+	return NODE_IDENTICAL;
+}
+
+
+static const struct operator_s* graph_optimize_operator_linop(const struct operator_s* op)
+{
+	if (NULL == op)
+		return NULL;
+
+	//assert(1 == operator_nr_out_args(op));
+	//assert(1 == operator_nr_in_args(op));
+	//assert(operator_get_io_flags(op)[0]);
+	//assert(!operator_get_io_flags(op)[1]);
+
+	auto graph = operator_get_graph(op);
+
+	int count1;
+	int count2 = list_count(graph->nodes);
+
+	do {
+		count1 = count2;
+
+		graph = operator_graph_optimize_identity_F(graph);
+		graph = operator_graph_sum_to_multi_sum_F(graph, false);
+		graph = operator_graph_optimize_identify_F(graph);
+		graph = operator_graph_optimize_linops_F(graph, node_identify_linop);
+
+		count2 = list_count(graph->nodes);
+
+	} while(count1 > count2);
+
+	return graph_to_operator_F(graph);
+}
+
+
+struct linop_s* graph_optimize_linop(const struct linop_s* op)
+{
+	PTR_ALLOC(struct linop_s, c);
+
+	c->forward = graph_optimize_operator_linop(op->forward);
+	c->adjoint = graph_optimize_operator_linop(op->adjoint);
+	c->normal = graph_optimize_operator_linop(op->normal);
+	c->norm_inv = operator_p_ref(op->norm_inv);
+
+	return PTR_PASS(c);
+}
+
+
+//FIXME: This is not optimal as it should be part of the operator framework only.
+//However, to optimize using Ax + Ay = A(x+y) the information of the linop framework is necessary.
+void operator_linops_apply_joined_unchecked(unsigned int N, const struct operator_s* op[N], complex float* dst[N], const complex float* src)
+{
+	auto combi = operator_combi_create(N, op);
+
+	int perm[2 * N];
+
+	for (int i = 0; i < (int)N; i++) {
+
+		perm[i] = 2 * i;
+		perm[i + N] = 2 * i + 1;
+	}
+
+	auto dup = operator_permute(combi, 2 * N, perm);
+
+	operator_free(combi);
+
+	for (int i = 0; i < (int)N - 1; i++) {
+
+		auto tmp = operator_dup_create(dup, N, N + 1);
+
+		operator_free(dup);
+
+		dup = tmp;
+	}
+
+	void* args[N + 1];
+
+	for (int i = 0; i < (int)N; i++)
+		args[i] = dst[i];
+
+	args[N] = (void*)src;
+
+	auto op_optimized = graph_optimize_operator_linop(dup);
+
+	operator_free(dup);
+
+	operator_generic_apply_unchecked(op_optimized, N + 1, args);
+
+	operator_free(op_optimized);
+}
+
