@@ -17,7 +17,7 @@
 #include "png.h"
 
 
-static int png_write_anyrgb(const char* name, unsigned int w, unsigned int h, unsigned int nbytes, bool rgb, const unsigned char* buf)
+static int png_write_anyrgb(const char* name, int w, int h, int nbytes, bool rgb, const unsigned char* buf)
 {
 
 	FILE* fp;
@@ -58,7 +58,7 @@ static int png_write_anyrgb(const char* name, unsigned int w, unsigned int h, un
 	row_ptrs = xmalloc(sizeof(png_bytep) * h);
 	int row_size = png_get_rowbytes(structp, infop);
 
-	for (unsigned int i = 0; i < h; i++)
+	for (int i = 0; i < h; i++)
 		row_ptrs[i] = (png_bytep)(buf + row_size * i);
 
 	png_write_image(structp, row_ptrs);
@@ -71,7 +71,7 @@ cleanup:
 		png_destroy_write_struct(&structp, &infop);
 
 	if (NULL != row_ptrs)
-		free(row_ptrs);
+		xfree(row_ptrs);
 
 	fclose(fp);
 	return ret;
@@ -82,25 +82,25 @@ cleanup:
 
 
 
-int png_write_rgb24(const char* name, unsigned int w, unsigned int h, long inum, const unsigned char* buf)
+int png_write_rgb24(const char* name, int w, int h, long inum, const unsigned char* buf)
 {
 	UNUSED(inum);
 	return png_write_anyrgb(name, w, h, 3, true, buf);
 }
 
-int png_write_rgb32(const char* name, unsigned int w, unsigned int h, long inum, const unsigned char* buf)
+int png_write_rgb32(const char* name, int w, int h, long inum, const unsigned char* buf)
 {
 	UNUSED(inum);
 	return png_write_anyrgb(name, w, h, 4, true, buf);
 }
 
-int png_write_bgr24(const char* name, unsigned int w, unsigned int h, long inum, const unsigned char* buf)
+int png_write_bgr24(const char* name, int w, int h, long inum, const unsigned char* buf)
 {
 	UNUSED(inum);
 	return png_write_anyrgb(name, w, h, 3, false, buf);
 }
 
-int png_write_bgr32(const char* name, unsigned int w, unsigned int h, long inum, const unsigned char* buf)
+int png_write_bgr32(const char* name, int w, int h, long inum, const unsigned char* buf)
 {
 	UNUSED(inum);
 	return png_write_anyrgb(name, w, h, 4, false, buf);
