@@ -2,6 +2,7 @@
 
 struct iter3_irgnm_conf;
 struct nlop_s;
+struct opt_reg_s;
 
 
 #ifndef DIMS
@@ -15,8 +16,16 @@ struct mdb_irgnm_l1_conf {
 
 	float step;
 	float lower_bound;
-	int constrained_maps;
-	bool auto_norm_off;
+	unsigned long constrained_maps;
+	unsigned long l2flags;
+	_Bool auto_norm;
+
+	int not_wav_maps;
+	unsigned int algo;
+	float rho;
+	struct opt_reg_s* ropts;
+	int tvscales_N;
+	complex float* tvscales;
 };
 
 void mdb_irgnm_l1(const struct mdb_irgnm_l1_conf* conf,
