@@ -13,7 +13,6 @@
 #include <complex.h>
 #include <stdbool.h>
 #include <assert.h>
-#include <stdio.h>
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
@@ -29,15 +28,23 @@
 #endif
 
 
-static const char usage_str[] = "<reference> <input>";
 static const char help_str[] = 
 	"Output normalized root mean square error (NRMSE),\n"
 	"i.e. norm(input - ref) / norm(ref)";
 			
 
 
-int main_nrmse(int argc, char* argv[])
+int main_nrmse(int argc, char* argv[argc])
 {
+	const char* ref_file = NULL;
+	const char* in_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &ref_file, "reference"),
+		ARG_INFILE(true, &in_file, "input"),
+	};
+
 	float test = -1.;
 	bool auto_scale = false;
 
@@ -47,15 +54,15 @@ int main_nrmse(int argc, char* argv[])
 		OPT_SET('s', &auto_scale, "automatic (complex) scaling"),
 	};
 
-	cmdline(&argc, argv, 2, 2, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
 	long ref_dims[DIMS];
 	long in_dims[DIMS];
 
-	complex float* ref = load_cfl(argv[1], DIMS, ref_dims);
-	complex float* in = load_cfl(argv[2], DIMS, in_dims);
+	complex float* ref = load_cfl(ref_file, DIMS, ref_dims);
+	complex float* in = load_cfl(in_file, DIMS, in_dims);
 
 	assert(md_check_compat(DIMS, 0u, in_dims, ref_dims));
 
@@ -77,12 +84,18 @@ int main_nrmse(int argc, char* argv[])
 
 	float err = md_znrmse(DIMS, ref_dims, ref, in);
 
-	printf("%f\n", err);
+	bart_printf("%f\n", err);
 
 	unmap_cfl(DIMS, ref_dims, ref);
 	unmap_cfl(DIMS, in_dims, in);
 
-	exit(((test == -1.) || (err <= test)) ? 0 : 1);
+	if (test == -1.)
+		test = err;
+
+	if (err < test * 0.1)
+		debug_printf(DP_DEBUG2, "Loose test: %f <= %f x %f\n", err, 0.1, test);
+
+	return (err <= test) ? 0 : 1;
 }
 
 
