@@ -33,7 +33,6 @@
 #include "noncart/precond.h"
 
 
-static const char usage_str[] = "<traj> <input> <output>";
 static const char help_str[] = "Perform non-uniform Fast Fourier Transform.";
 
 
@@ -42,6 +41,17 @@ static const char help_str[] = "Perform non-uniform Fast Fourier Transform.";
 
 int main_nufft(int argc, char* argv[argc])
 {
+	const char* traj_file = NULL;
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &traj_file, "traj"),
+		ARG_INFILE(true, &in_file, "input"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
 	bool adjoint = false;
 	bool inverse = false;
 	bool precond = false;
@@ -73,7 +83,7 @@ int main_nufft(int argc, char* argv[argc])
 		OPTL_SET(0, "lowmem", &conf.lowmem, "Use low-mem mode of the nuFFT"),
 	};
 
-	cmdline(&argc, argv, 3, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	// avoid computing PSF if not necessary
 	if (!inverse)
@@ -84,29 +94,40 @@ int main_nufft(int argc, char* argv[argc])
 
 	// Read trajectory
 	long traj_dims[DIMS];
-	complex float* traj = load_cfl(argv[1], DIMS, traj_dims);
+	complex float* traj = load_cfl(traj_file, DIMS, traj_dims);
 
 	assert(3 == traj_dims[0]);
 
+	long coilest_dims[DIMS];
+
+	estimate_im_dims(DIMS, FFT_FLAGS, coilest_dims, traj_dims, traj);
+
+	if (8 >= md_calc_size(3, coilest_dims)) {
+
+		debug_printf(DP_WARN,	"\tThe estimated image size %ldx%ldx%ld is very small.\n"
+					"\tDid you scale your trajectory correctly?\n"
+					"\tThe unit of measurement is pixel_size / FOV.\n",
+					coilest_dims[0], coilest_dims[1], coilest_dims[2]);
+	}
 
 	(use_gpu ? num_init_gpu : num_init)();
 
 	if (inverse || adjoint) {
 
 		long ksp_dims[DIMS];
-		const complex float* ksp = load_cfl(argv[2], DIMS, ksp_dims);
+		const complex float* ksp = load_cfl(in_file, DIMS, ksp_dims);
 
 		assert(1 == ksp_dims[0]);
 		assert(md_check_compat(DIMS, ~(PHS1_FLAG|PHS2_FLAG), ksp_dims, traj_dims));
 
 		if (0 == md_calc_size(3, coilim_dims)) {
 
-			estimate_im_dims(DIMS, FFT_FLAGS, coilim_dims, traj_dims, traj);
-			debug_printf(DP_INFO, "Est. image size: %ld %ld %ld\n", coilim_dims[0], coilim_dims[1], coilim_dims[2]);
+			md_copy_dims(DIMS, coilim_dims, coilest_dims);
+			debug_printf(DP_INFO, "Est. image size: %ldx%ldx%ld\n", coilim_dims[0], coilim_dims[1], coilim_dims[2]);
 
 			if (!conf.decomp) {
 
-				for (unsigned int i = 0; i < DIMS; i++)
+				for (int i = 0; i < DIMS; i++)
 					if (MD_IS_SET(FFT_FLAGS, i) && (1 < coilim_dims[i]))
 						coilim_dims[i] *= 2;
 			}
@@ -114,7 +135,7 @@ int main_nufft(int argc, char* argv[argc])
 
 		md_copy_dims(DIMS - 3, coilim_dims + 3, ksp_dims + 3);
 
-		complex float* img = create_cfl(argv[3], DIMS, coilim_dims);
+		complex float* img = create_cfl(out_file, DIMS, coilim_dims);
 
 		md_clear(DIMS, coilim_dims, img, CFL_SIZE);
 
@@ -155,14 +176,14 @@ int main_nufft(int argc, char* argv[argc])
 	} else {
 
 		// Read image data
-		const complex float* img = load_cfl(argv[2], DIMS, coilim_dims);
+		const complex float* img = load_cfl(in_file, DIMS, coilim_dims);
 
 		// Initialize kspace data
 		long ksp_dims[DIMS];
 		md_select_dims(DIMS, PHS1_FLAG|PHS2_FLAG, ksp_dims, traj_dims);
 		md_copy_dims(DIMS - 3, ksp_dims + 3, coilim_dims + 3);
 
-		complex float* ksp = create_cfl(argv[3], DIMS, ksp_dims);
+		complex float* ksp = create_cfl(out_file, DIMS, ksp_dims);
 
 		const struct linop_s* nufft_op;
 
@@ -175,6 +196,7 @@ int main_nufft(int argc, char* argv[argc])
 		linop_forward(nufft_op, DIMS, ksp_dims, ksp, DIMS, coilim_dims, img);
 
 		linop_free(nufft_op);
+
 		unmap_cfl(DIMS, coilim_dims, img);
 		unmap_cfl(DIMS, ksp_dims, ksp);
 	}
@@ -182,6 +204,7 @@ int main_nufft(int argc, char* argv[argc])
 	unmap_cfl(DIMS, traj_dims, traj);
 
 	debug_printf(DP_INFO, "Done.\n");
+
 	return 0;
 }
 
