@@ -190,54 +190,69 @@ struct fft_plan_s {
 
 	fftwf_plan fftw;
 
-#ifdef  USE_CUDA
-#ifdef	LAZY_CUDA
 	unsigned int D;
 	unsigned long flags;
 	bool backwards;
 	const long* dims;
 	const long* istrs;
 	const long* ostrs;
-#endif
+
+#ifdef  USE_CUDA
 	struct fft_cuda_plan_s* cuplan;
 #endif
 };
 
 static DEF_TYPEID(fft_plan_s);
 
-static char* fftw_wisdom_name(unsigned int N, bool backwards, unsigned int flags, const long dims[N])
-{
 
+#ifdef USE_FFTW_WISDOM
+static char* fftw_wisdom_name(int N, bool backwards, unsigned int flags, const long dims[N])
+{
 	char* tbpath = getenv("TOOLBOX_PATH");
+
 	if (NULL == tbpath)
 		return NULL;
-	char* loc  = NULL;
 
 	// Space for path and null terminator.
-	size_t space = snprintf(loc, 0, "%s/save/fftw/N_%d_BACKWARD_%d_FLAGS_%d_DIMS", tbpath, N, backwards, flags);
+	int space = snprintf(NULL, 0, "%s/save/fftw/N_%d_BACKWARD_%d_FLAGS_%d_DIMS", tbpath, N, backwards, flags);
+
 	// Space for dimensions.
-	for (size_t idx = 0; idx < N; idx ++)
-		space += snprintf(loc, 0, "_%lu", dims[idx]);
+	for (int idx = 0; idx < N; idx ++)
+		space += snprintf(NULL, 0, "_%lu", dims[idx]);
+
 	// Space for extension.
-	space += snprintf(loc, 0, ".fftw");
+	space += snprintf(NULL, 0, ".fftw");
 	// Space for null terminator.
 	space += 1;
 
-	loc = calloc(space, sizeof(char));
-	sprintf(loc , "%s/save/fftw/N_%d_BACKWARD_%d_FLAGS_%d_DIMS", tbpath, N, backwards, flags);
+	int len = space;
+	char* loc = calloc(space, sizeof(char));
 
-	char tmp[64];
-	for (size_t idx = 0; idx < N; idx++) {
-		sprintf(tmp, "_%lu", dims[idx]);
+	if (NULL == loc)
+		error("memory out");
+
+	int ret = snprintf(loc, len, "%s/save/fftw/N_%d_BACKWARD_%d_FLAGS_%d_DIMS", tbpath, N, backwards, flags);
+
+	assert(ret < len);
+	len -= ret;
+
+	for (int idx = 0; idx < N; idx++) {
+
+		char tmp[64];
+		ret = sprintf(tmp, "_%lu", dims[idx]);
+		assert(ret < 64);
+		len -= ret;
 		strcat(loc, tmp);
 	}
 
-	sprintf(tmp, ".fftw");
-	strcat(loc, tmp);
-	loc[space - 1] = '\0';
+	strcat(loc, ".fftw");
+	len -= 5;
+	assert(1 == len);
+	assert('\0' == loc[space - 1]);
 
 	return loc;
 }
+#endif //USE_FFTW_WISDOM
 
 static fftwf_plan fft_fftwf_plan(unsigned int D, const long dimensions[D], unsigned long flags, const long ostrides[D], complex float* dst, const long istrides[D], const complex float* src, bool backwards, bool measure)
 {
@@ -249,10 +264,14 @@ static fftwf_plan fft_fftwf_plan(unsigned int D, const long dimensions[D], unsig
 	unsigned int k = 0;
 	unsigned int l = 0;
 
+#ifdef USE_FFTW_WISDOM
 	char* wisdom = fftw_wisdom_name(D, backwards, flags, dimensions);
+
 	if (NULL != wisdom)
 		fftwf_import_wisdom_from_filename(wisdom);
 
+#endif //USE_FFTW_WISDOM
+
 	//FFTW seems to be fine with this
 	//assert(0 != flags); 
 
@@ -278,10 +297,12 @@ static fftwf_plan fft_fftwf_plan(unsigned int D, const long dimensions[D], unsig
 	fftwf = fftwf_plan_guru64_dft(k, dims, l, hmdims, (complex float*)src, dst,
 				backwards ? 1 : (-1), measure ? FFTW_MEASURE : FFTW_ESTIMATE);
 
+#ifdef USE_FFTW_WISDOM
 	if (NULL != wisdom)
 		fftwf_export_wisdom_to_filename(wisdom);
 
 	md_free(wisdom);
+#endif //USE_FFTW_WISDOM
 
 	return fftwf;
 }
@@ -295,13 +316,21 @@ static void fft_apply(const operator_data_t* _plan, unsigned int N, void* args[N
 
 	assert(2 == N);
 
+	if (0u == plan->flags) {
+
+		md_copy2(plan->D, plan->dims, plan->ostrs, dst, plan->istrs, src, CFL_SIZE);
+		return;
+	}
+
 #ifdef  USE_CUDA
 	if (cuda_ondevice(src)) {
 #ifdef	LAZY_CUDA
-          if (NULL == plan->cuplan)
-		((struct fft_plan_s*)plan)->cuplan = fft_cuda_plan(plan->D, plan->dims, plan->flags, plan->ostrs, plan->istrs, plan->backwards);
+		if (NULL == plan->cuplan)
+			((struct fft_plan_s*)plan)->cuplan = fft_cuda_plan(plan->D, plan->dims, plan->flags, plan->ostrs, plan->istrs, plan->backwards);
 #endif
-		assert(NULL != plan->cuplan);
+		if (NULL == plan->cuplan)
+			error("Failed to plan a GPU FFT (too large?)\n");
+
 		fft_cuda_exec(plan->cuplan, dst, src);
 
 	} else 
@@ -317,22 +346,25 @@ static void fft_free_plan(const operator_data_t* _data)
 {
 	const auto plan = CAST_DOWN(fft_plan_s, _data);
 
-	fftwf_destroy_plan(plan->fftw);
+	if (NULL != plan->fftw)
+		fftwf_destroy_plan(plan->fftw);
+
 #ifdef	USE_CUDA
-#ifdef	LAZY_CUDA
-	xfree(plan->dims);
-	xfree(plan->istrs);
-	xfree(plan->ostrs);
-#endif
 	if (NULL != plan->cuplan)
 		fft_cuda_free_plan(plan->cuplan);
 #endif
+	xfree(plan->dims);
+	xfree(plan->istrs);
+	xfree(plan->ostrs);
+
 	xfree(plan);
 }
 
 
 const struct operator_s* fft_measure_create(unsigned int D, const long dimensions[D], unsigned long flags, bool inplace, bool backwards)
 {
+	flags &= md_nontriv_dims(D, dimensions);
+
 	PTR_ALLOC(struct fft_plan_s, plan);
 	SET_TYPEID(fft_plan_s, plan);
 
@@ -342,7 +374,10 @@ const struct operator_s* fft_measure_create(unsigned int D, const long dimension
 	long strides[D];
 	md_calc_strides(D, strides, dimensions, CFL_SIZE);
 
-	plan->fftw = fft_fftwf_plan(D, dimensions, flags, strides, dst, strides, src, backwards, true);
+	plan->fftw = NULL;
+
+	if (0u != flags)
+		plan->fftw = fft_fftwf_plan(D, dimensions, flags, strides, dst, strides, src, backwards, true);
 
 	md_free(src);
 
@@ -352,9 +387,10 @@ const struct operator_s* fft_measure_create(unsigned int D, const long dimension
 #ifdef  USE_CUDA
 	plan->cuplan = NULL;
 #ifndef LAZY_CUDA
-	if (cuda_ondevice(src))
-          plan->cuplan = fft_cuda_plan(D, dimensions, flags, strides, strides, backwards);
-#else
+	if (cuda_ondevice(src) && (0u != flags)
+		plan->cuplan = fft_cuda_plan(D, dimensions, flags, strides, strides, backwards);
+#endif
+#endif
 	plan->D = D;
 	plan->flags = flags;
 	plan->backwards = backwards;
@@ -370,25 +406,30 @@ const struct operator_s* fft_measure_create(unsigned int D, const long dimension
 	PTR_ALLOC(long[D], ostrs);
 	md_copy_strides(D, *ostrs, strides);
 	plan->ostrs = *PTR_PASS(ostrs);
-#endif
-#endif
+
 	return operator_create2(D, dimensions, strides, D, dimensions, strides, CAST_UP(PTR_PASS(plan)), fft_apply, fft_free_plan);
 }
 
 
 const struct operator_s* fft_create2(unsigned int D, const long dimensions[D], unsigned long flags, const long ostrides[D], complex float* dst, const long istrides[D], const complex float* src, bool backwards)
 {
+	flags &= md_nontriv_dims(D, dimensions);
+
 	PTR_ALLOC(struct fft_plan_s, plan);
 	SET_TYPEID(fft_plan_s, plan);
 
-	plan->fftw = fft_fftwf_plan(D, dimensions, flags, ostrides, dst, istrides, src, backwards, false);
+	plan->fftw = NULL;
+
+	if (0u != flags)
+		plan->fftw = fft_fftwf_plan(D, dimensions, flags, ostrides, dst, istrides, src, backwards, false);
 
 #ifdef  USE_CUDA
 	plan->cuplan = NULL;
 #ifndef LAZY_CUDA
-	if (cuda_ondevice(src))
+	if (cuda_ondevice(src) && (0u != flags)
 		plan->cuplan = fft_cuda_plan(D, dimensions, flags, ostrides, istrides, backwards);
-#else
+#endif
+#endif
 	plan->D = D;
 	plan->flags = flags;
 	plan->backwards = backwards;
@@ -404,8 +445,6 @@ const struct operator_s* fft_create2(unsigned int D, const long dimensions[D], u
 	PTR_ALLOC(long[D], ostrs);
 	md_copy_strides(D, *ostrs, ostrides);
 	plan->ostrs = *PTR_PASS(ostrs);
-#endif
-#endif
 
 	return operator_create2(D, dimensions, ostrides, D, dimensions, istrides, CAST_UP(PTR_PASS(plan)), fft_apply, fft_free_plan);
 }
