@@ -5,6 +5,9 @@
 
 extern void num_init(void);
 extern void num_init_gpu(void);
+extern void num_deinit_gpu(void);
+extern void num_init_multigpu_select(unsigned long requested_gpus);
+extern void num_init_multigpu(int requested_gpus);
 /**
  * selects the GPU with the maximum available memory
  *   (if there are more than one on the system)
