@@ -31,7 +31,13 @@
 #include <string.h>
 #include <assert.h>
 #include <stdbool.h>
+
+#ifdef _WIN32
+#include <malloc.h>
+#else
 #include <alloca.h>
+#endif
+
 #include <strings.h>
 
 #include "misc/misc.h"
@@ -83,6 +89,8 @@ void md_nary(unsigned int C, unsigned int D, const long dim[D], const long* str[
  */
 void md_parallel_nary(unsigned int C, unsigned int D, const long dim[D], unsigned long flags, const long* str[C], void* ptr[C], md_nary_fun_t fun)
 {
+	flags = flags & md_nontriv_dims(D, dim);
+
 	if (0 == flags) {
 
 		md_nary(C, D, dim, str, ptr, fun);
@@ -159,6 +167,7 @@ static void md_parallel_loop_r(unsigned int D, unsigned int N, const long dim[st
 	// we need to make a copy because firstprivate needs to see
 	// an array instead of a pointer
 	long pos_copy[N];
+
 	for (unsigned int i = 0; i < N; i++)
 		pos_copy[i] = pos[i];
 
@@ -335,7 +344,7 @@ void md_select_dims(unsigned int D, unsigned long flags, long odims[D], const lo
  */
 void md_copy_dims(unsigned int D, long odims[D], const long idims[D])
 {
-	memcpy(odims, idims, D  * sizeof(long));
+	memcpy(odims, idims, D * sizeof(long));
 }
 
 
@@ -481,7 +490,7 @@ bool md_check_compat(unsigned int D, unsigned long flags, const long dim1[D], co
 
 void md_merge_dims(unsigned int N, long out_dims[N], const long dims1[N], const long dims2[N])
 {
-	assert(md_check_compat(N, ~0, dims1, dims2));
+	assert(md_check_compat(N, ~0UL, dims1, dims2));
 
 	for (unsigned int i = 0; i < N; i++)
 		out_dims[i] = (1 == dims1[i]) ? dims2[i] : dims1[i];
@@ -644,6 +653,7 @@ void md_copy2(unsigned int D, const long dim[D], const long ostr[D], void* optr,
 
 #ifdef	USE_CUDA
 	bool use_gpu = cuda_ondevice(optr) || cuda_ondevice(iptr);
+
 #if 1
 	long tostr[D];
 	long tistr[D];
@@ -654,16 +664,95 @@ void md_copy2(unsigned int D, const long dim[D], const long ostr[D], void* optr,
 	md_copy_dims(D, tdims, dim);
 
 	long (*nstr2[2])[D] = { &tostr, &tistr };
-	int ND = optimize_dims(2, D, tdims, nstr2);
+	int ND = optimize_dims_gpu(2, D, tdims, nstr2);
 
+	assert(ND <= (int)D);
+
+#if 1
+	// permute dims with 0 input strides or negative in/output strides to the end
+	// these might be permuted to the inner dimensions by optimize_dims and break the strided copy
+
+	unsigned int perm[ND];
+
+	for (int i = 0, j = 0; i < ND; i++) {
+
+		if (   (0 >= (*nstr2[1])[i])
+		    || (0 >= (*nstr2[0])[i])) {
+
+			perm[ND - 1 -j] = i;
+			j += 1;
+
+		} else {
+
+			perm[i - j] = i;
+		}
+	}
+
+	long tmp[ND];
+
+	md_permute_dims(ND, perm, tmp, tdims);
+	md_copy_dims(ND, tdims, tmp);
+
+	md_permute_dims(ND, perm, tmp, tostr);
+	md_copy_dims(ND, tostr, tmp);
+
+	md_permute_dims(ND, perm, tmp, tistr);
+	md_copy_dims(ND, tistr, tmp);
+#endif
+
+#if 1
+	//fill like copies
+
+	unsigned long fill_flags =  md_nontriv_dims(D, tdims)
+				 & ~md_nontriv_strides(D, tistr)
+				 & md_nontriv_strides(D, tostr);
+
+	if (use_gpu && (0 != fill_flags)) {
+
+		int idx = md_min_idx(fill_flags);
+
+		long tdims2[ND];
+		long pos[ND];
+
+		md_select_dims(ND, ~MD_BIT(idx), tdims2, tdims);
+		md_singleton_strides(ND, pos);
+
+		md_copy2(ND, tdims2, tostr, optr, tistr, iptr, size);
+
+		pos[idx] = 1;
+
+		while (pos[idx] < tdims[idx]) {
+
+			tdims2[idx] = MIN(pos[idx], tdims[idx] - pos[idx]);
+
+			md_copy2(ND, tdims2, tostr, optr + md_calc_offset(ND, tostr, pos), tostr, optr, size);
+
+			pos[idx] += tdims2[idx];
+		}
+
+		return;
+	}
+#endif
 
 	size_t sizes[2] = { size, size };
 	int skip = min_blockdim(2, ND, tdims, nstr2, sizes);
 
-	long ostr2 = (*nstr2[0])[skip];
-	long istr2 = (*nstr2[1])[skip];
+	debug_printf(DP_DEBUG4, "md_copy_2 skip=%d\n", skip);
+	debug_print_dims(DP_DEBUG4, ND, tdims);
+	debug_print_dims(DP_DEBUG4, ND, (*nstr2[0]));
+	debug_print_dims(DP_DEBUG4, ND, (*nstr2[1]));
 
-	if (use_gpu && (ND - skip > 0) && (ostr2 > 0) && (istr2 > 0)) {
+	if (   use_gpu
+	    && (ND - skip > 0)) {
+
+		assert(skip < ND);
+
+		long ostr2 = (*nstr2[0])[skip];
+		long istr2 = (*nstr2[1])[skip];
+
+		if (!(   (ostr2 > 0)
+	              && (istr2 > 0)))
+			goto out;
 
 		void* nptr[2] = { optr, (void*)iptr };
 		long sizes[2] = { md_calc_size(skip, tdims) * size, tdims[skip] };
@@ -673,11 +762,12 @@ void md_copy2(unsigned int D, const long dim[D], const long ostr[D], void* optr,
 		const long* nstr[2] = { *nstr2[0] + skip, *nstr2[1] + skip };
 
 		long* sizesp = sizes; // because of clang
+		void** nptrp = nptr;
 
 		NESTED(void, nary_strided_copy, (void* ptr[]))
 		{
 			debug_printf(DP_DEBUG4, "CUDA 2D copy %ld %ld %ld %ld %ld %ld\n",
-				sizes[0], sizes[1], ostr2, istr2, nptr[0], nptr[1]);
+				sizesp[0], sizesp[1], ostr2, istr2, nptrp[0], nptrp[1]);
 
 			cuda_memcpy_strided(sizesp, ostr2, ptr[0], istr2, ptr[1]);
 		};
@@ -685,6 +775,8 @@ void md_copy2(unsigned int D, const long dim[D], const long ostr[D], void* optr,
 		md_nary(2, ND - skip, tdims + skip, nstr, nptr, nary_strided_copy);
 		return;
 	}
+
+out:	;
 #endif
 #endif
 	const long (*nstr[2])[D] = { (const long (*)[D])ostr, (const long (*)[D])istr };
@@ -778,10 +870,12 @@ void md_fill(unsigned int D, const long dim[D], void* ptr, const void* iptr, siz
 void md_circular_swap2(unsigned int M, unsigned int D, const long dims[D], const long* strs[M], void* ptr[M], size_t size)
 {
 	size_t sizes[M];
+
 	for (unsigned int i = 0; i < M; i++)
 		sizes[i] = size;
 
 	const long (*nstrs[M])[D];
+
 	for (unsigned int i = 0; i < M; i++)
 		nstrs[i] = (const long (*)[D])strs[i];
 
@@ -1008,8 +1102,8 @@ void md_resize_center(unsigned int D, const long odim[D], void* optr, const long
  */
 void md_pad_center(unsigned int D, const void* val, const long odim[D], void* optr, const long idim[D], const void* iptr, size_t size)
 {
-
 	long pos[D];
+
 	for (unsigned int i = 0; i < D; i++)
 		pos[i] = labs((odim[i] / 2) - (idim[i] / 2));
 
@@ -1187,6 +1281,7 @@ void md_swap_flip2(unsigned int D, const long dims[D], unsigned long flags, cons
 {
 #if 1
 	int i;
+
 	for (i = D - 1; i >= 0; i--)
 		if ((1 != dims[i]) && MD_IS_SET(flags, i))
 			break;
@@ -1202,10 +1297,13 @@ void md_swap_flip2(unsigned int D, const long dims[D], unsigned long flags, cons
 	assert(istr[i] != 0);
 
 	long dims2[D];
+
 	md_copy_dims(D, dims2, dims);
+
 	dims2[i] = dims[i] / 2;
 
 	long off = (dims[i] + 1) / 2;
+
 	assert(dims2[i] + off == dims[i]);
 
 	md_swap_flip2(D, dims2, flags, ostr, optr, istr, iptr + off * istr[i], size);
@@ -1246,6 +1344,7 @@ void md_swap_flip(unsigned int D, const long dims[D], unsigned long flags, void*
 static void md_flip_inpl2(unsigned int D, const long dims[D], unsigned long flags, const long str[D], void* ptr, size_t size)
 {
 	int i;
+
 	for (i = D - 1; i >= 0; i--)
 		if ((1 != dims[i]) && MD_IS_SET(flags, i))
 			break;
@@ -1257,7 +1356,9 @@ static void md_flip_inpl2(unsigned int D, const long dims[D], unsigned long flag
 	assert(str[i] != 0);
 
 	long dims2[D];
+
 	md_copy_dims(D, dims2, dims);
+
 	dims2[i] = dims[i] / 2;
 
 	long off = str[i] * (0 + (dims[i] + 1) / 2);
@@ -1410,7 +1511,7 @@ bool md_compare2(unsigned int D, const long dims[D], const long str1[D], const v
 
 		bool eq2 = (0 == memcmp(ptrs[0], ptrs[1], size2));
 
-		#pragma omp critical
+		#pragma omp atomic
 		eq &= eq2;
 	};
 
@@ -1632,6 +1733,7 @@ void md_circ_shift2(unsigned int D, const long dimensions[D], const long center[
 	}
 
 	unsigned int i = 0;		// FIXME :maybe we shoud search the other way?
+
 	while ((i < D) && (0 == pos[i]))
 		i++;
 
@@ -1870,6 +1972,19 @@ void* md_alloc_sameplace(unsigned int D, const long dimensions[D], size_t size,
 #endif
 }
 
+/**
+ * Check whether memory is at sameplace
+ */
+bool md_is_sameplace(const void* ptr1, const void* ptr2)
+{
+	assert(NULL != ptr1);
+	assert(NULL != ptr2);
+#ifdef USE_CUDA
+	return cuda_ondevice(ptr1) == cuda_ondevice(ptr2);
+#else
+	return true;
+#endif
+}
 
 
 /**
@@ -1885,3 +2000,20 @@ void md_free(const void* ptr)
 #endif
 	xfree(ptr);
 }
+
+
+int md_max_idx(unsigned long flags)
+{
+	int i = -1;
+
+	for ( ; 0 != flags; i++)
+		flags /= 2;
+
+	return i;
+}
+
+int md_min_idx(unsigned long flags)
+{
+	return ffsl(flags) - 1;
+}
+
