@@ -1,10 +1,9 @@
 /* Copyright 2013. The Regents of the University of California.
- * All rights reserved. Use of this source code is governed by 
+ * Copyright 2021. Uecker Lab. University Center Göttingen.
+ * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
- * Authors:
- * 2013 Martin Uecker <uecker@eecs.berkeley.edu>
- * 2013 Dara Bahri <dbahri123@gmail.com>
+ * Authors: Martin Uecker, Dara Bahri, Moritz Blumenthal
  */
 
 #define _GNU_SOURCE
@@ -12,6 +11,10 @@
 #include <math.h>
 #include <complex.h>
 
+#ifdef _WIN32
+#include "win/rand_r.h"
+#endif
+
 #include "num/multind.h"
 
 #ifdef USE_CUDA
@@ -48,7 +51,7 @@ complex double gaussian_rand(void)
 	double u1, u2, s;
 
  	do {
-	
+
 		u1 = 2. * uniform_rand() - 1.;
 		u2 = 2. * uniform_rand() - 1.;
    		s = u1 * u1 + u2 * u2;
@@ -65,7 +68,7 @@ void md_gaussian_rand(unsigned int D, const long dims[D], complex float* dst)
 {
 #ifdef  USE_CUDA
 	if (cuda_ondevice(dst)) {
-	
+
 		complex float* tmp = md_alloc(D, dims, sizeof(complex float));
 		md_gaussian_rand(D, dims, tmp);
 		md_copy(D, dims, dst, tmp, sizeof(complex float));
@@ -78,3 +81,34 @@ void md_gaussian_rand(unsigned int D, const long dims[D], complex float* dst)
 		dst[i] = (complex float)gaussian_rand();
 }
 
+void md_uniform_rand(unsigned int D, const long dims[D], complex float* dst)
+{
+#ifdef  USE_CUDA
+	if (cuda_ondevice(dst)) {
+
+		complex float* tmp = md_alloc(D, dims, sizeof(complex float));
+		md_uniform_rand(D, dims, tmp);
+		md_copy(D, dims, dst, tmp, sizeof(complex float));
+		md_free(tmp);
+		return;
+	}
+#endif
+	for (long i = 0; i < md_calc_size(D, dims); i++)
+		dst[i] = (complex float)uniform_rand();
+}
+
+void md_rand_one(unsigned int D, const long dims[D], complex float* dst, double p)
+{
+#ifdef  USE_CUDA
+	if (cuda_ondevice(dst)) {
+
+		complex float* tmp = md_alloc(D, dims, sizeof(complex float));
+		md_rand_one(D, dims, tmp, p);
+		md_copy(D, dims, dst, tmp, sizeof(complex float));
+		md_free(tmp);
+		return;
+	}
+#endif
+	for (long i = 0; i < md_calc_size(D, dims); i++)
+		dst[i] = (complex float)(uniform_rand() < p);
+}
