@@ -24,13 +24,19 @@
 
 
 
-static const char usage_str[] = "<output>";
 static const char help_str[] = "Image and k-space domain phantoms.";
 
 
 
 int main_phantom(int argc, char* argv[argc])
 {
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
 	bool kspace = false;
 	bool d3 = false;
 	int sens = 0;
@@ -38,9 +44,10 @@ int main_phantom(int argc, char* argv[argc])
 	int xdim = -1;
 
 	int geo = -1;
-	enum ptype_e { SHEPPLOGAN, CIRC, TIME, SENS, GEOM, STAR, BART, TUBES, RAND_TUBES } ptype = SHEPPLOGAN;
 
-	const char* traj = NULL;
+	enum ptype_e { SHEPPLOGAN, CIRC, TIME, SENS, GEOM, STAR, BART, TUBES, RAND_TUBES, NIST, SONAR } ptype = SHEPPLOGAN;
+
+	const char* traj_file = NULL;
 	bool basis = false;
 
 	long dims[DIMS] = { [0 ... DIMS - 1] = 1 };
@@ -51,28 +58,35 @@ int main_phantom(int argc, char* argv[argc])
 	int rinit = -1;
 	int N = -1;
 
+	float rotation_angle = 0.;
+	int rotation_steps = 1;
+
 
 	const struct opt_s opts[] = {
 
 		OPT_INT('s', &sens, "nc", "nc sensitivities"),
 		OPT_INT('S', &osens, "nc", "Output nc sensitivities"),
 		OPT_SET('k', &kspace, "k-space"),
-		OPT_STRING('t', &traj, "file", "trajectory"),
+		OPT_INFILE('t', &traj_file, "file", "trajectory"),
 		OPT_SELECT('c', enum ptype_e, &ptype, CIRC, "()"),
 		OPT_SELECT('a', enum ptype_e, &ptype, STAR, "()"),
 		OPT_SELECT('m', enum ptype_e, &ptype, TIME, "()"),
 		OPT_SELECT('G', enum ptype_e, &ptype, GEOM, "geometric object phantom"),
 		OPT_SELECT('T', enum ptype_e, &ptype, TUBES, "tubes phantom"),
+		OPTL_SELECT(0, "NIST", enum ptype_e, &ptype, NIST, "NIST phantom (T2 sphere)"),
+                OPTL_SELECT(0, "SONAR", enum ptype_e, &ptype, SONAR, "Diagnostic Sonar phantom"),
 		OPT_INT('N', &N, "num", "Random tubes phantom and number"),
 		OPT_SELECT('B', enum ptype_e, &ptype, BART, "BART logo"),
 		OPT_INT('x', &xdim, "n", "dimensions in y and z"),
-		OPT_INT('g', &geo, "n=1,2", "select geometry for object phantom"),
+		OPT_INT('g', &geo, "n=1,2,3", "select geometry for object phantom"),
 		OPT_SET('3', &d3, "3D"),
 		OPT_SET('b', &basis, "basis functions for geometry"),
 		OPT_INT('r', &rinit, "seed", "random seed initialization"),
+		OPTL_FLOAT(0, "rotation-angle", &(rotation_angle), "[deg]", "Angle of Rotation"),
+		OPTL_INT(0, "rotation-steps", &(rotation_steps), " ", "Number of rotation steps"),
 	};
 
-	cmdline(&argc, argv, 1, 1, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
@@ -84,9 +98,11 @@ int main_phantom(int argc, char* argv[argc])
 		ptype = RAND_TUBES;
 		if (N > 200)
 			BART_WARN("Number of tubes is large. Runetime may be very slow.\n");
+
+	} else {
+
+		N = (SONAR == ptype ? 8 : (NIST == ptype ? 15 : 11));
 	}
-	else
-		N = 11;
 
 	if ((GEOM != ptype) && (-1 != geo)) {
 
@@ -101,6 +117,9 @@ int main_phantom(int argc, char* argv[argc])
 	if (TIME == ptype)
 		dims[TE_DIM] = 32;
 
+	if ((TUBES == ptype) || (NIST == ptype) || (SONAR == ptype))
+		dims[TIME_DIM] = rotation_steps;
+
 	if (-1 != osens) {
 
 		assert(SHEPPLOGAN == ptype);
@@ -119,14 +138,14 @@ int main_phantom(int argc, char* argv[argc])
 	long sstrs[DIMS] = { 0 };
 	complex float* samples = NULL;
 
-	if (NULL != traj) {
+	if (NULL != traj_file) {
 
 		if (-1 != xdim)
 			debug_printf(DP_WARN, "size ignored.\n");
 
 		kspace = true;
 
-		samples = load_cfl(traj, DIMS, sdims);
+		samples = load_cfl(traj_file, DIMS, sdims);
 
 		md_calc_strides(DIMS, sstrs, sdims, sizeof(complex float));
 
@@ -144,12 +163,12 @@ int main_phantom(int argc, char* argv[argc])
 
 	if (basis) {
 
-		assert(TUBES == ptype || RAND_TUBES == ptype);
+		assert(TUBES == ptype || RAND_TUBES == ptype || NIST == ptype || SONAR == ptype);
 		dims[COEFF_DIM] = N; // Number of elements of tubes phantom with rings see src/shepplogan.c
 	}
 
 
-	complex float* out = create_cfl(argv[1], DIMS, dims);
+	complex float* out = create_cfl(out_file, DIMS, dims);
 
 	md_clear(DIMS, dims, out, sizeof(complex float));
 
@@ -158,7 +177,7 @@ int main_phantom(int argc, char* argv[argc])
 
 	case SENS:
 
-		assert(NULL == traj);
+		assert(NULL == traj_file);
 		assert(!kspace);
 
 		calc_sens(dims, out);
@@ -166,7 +185,7 @@ int main_phantom(int argc, char* argv[argc])
 
 	case GEOM:
 
-		if ((geo < 1) || (geo > 2))
+		if ((geo < 1) || (geo > 3))
 			error("geometric phantom: invalid geometry");
 
 		if (d3)
@@ -199,13 +218,15 @@ int main_phantom(int argc, char* argv[argc])
 		break;
 
 	case TUBES:
+	case NIST:
+        case SONAR:
 
-		calc_phantom_tubes(dims, out, kspace, false, N, sstrs, samples);
+		calc_phantom_tubes(dims, out, kspace, false, rotation_angle, N, sstrs, samples);
 		break;
 
 	case RAND_TUBES:
 
-		calc_phantom_tubes(dims, out, kspace, true, N, sstrs, samples);
+		calc_phantom_tubes(dims, out, kspace, true, rotation_angle, N, sstrs, samples);
 		break;
 
 	case BART:
@@ -214,8 +235,6 @@ int main_phantom(int argc, char* argv[argc])
 		break;
 	}
 
-	xfree(traj);
-
 	if (NULL != samples)
 		unmap_cfl(3, sdims, samples);
 
