@@ -28,14 +28,14 @@
 #endif
 
 
-static const char usage_str[] = "<input>";
 static const char help_str[] = "Outputs values or meta data.";
 
 
-static void print_cfl(unsigned int N, const long dims[N], const complex float* data, const char* fmt, const char* sep)
+static void print_cfl(int N, const long dims[N], const complex float* data, const char* fmt, const char* sep)
 {
 	// find first non-trivial dimension
-	unsigned int l = 0;
+	int l = 0;
+
 	while ((l < N - 1) && (1 == dims[l]))
 		l++;
 
@@ -43,20 +43,24 @@ static void print_cfl(unsigned int N, const long dims[N], const complex float* d
 
 	const char* allowed_fmts[] = {
 
-	 	"%%+%[0-9.]f%%+%[0-9.]fi",
-		"%%+%[0-9.]e%%+%[0-9.]ei"
+		"%%+%*[0-9.]f%%+%*[0-9.]fi%n",
+		"%%+%*[0-9.]e%%+%*[0-9.]ei%n",
+		"%%+%*[0-9.]f,%%+%*[0-9.]f%n",
+		"%%+%*[0-9.]e,%%+%*[0-9.]e%n",
 	};
 
-	// declare buffers for the real and imaginary format values
-	char fmt_re[strlen(fmt)];
-	char fmt_im[strlen(fmt)];
-
 	// ensure that the input format string matches one of the valid format templates
-	for (unsigned int i = 0; i < ARRAY_SIZE(allowed_fmts); i++)
-		if (2 == sscanf(fmt, allowed_fmts[i], &fmt_re, &fmt_im))
-			goto ok;
+	for (int i = 0; i < (int)ARRAY_SIZE(allowed_fmts); i++) {
+
+		size_t rd = 0;
+
+		if (0 == sscanf(fmt, allowed_fmts[i], &rd))
+			if (strlen(fmt) == rd)
+				goto ok;
+	}
 
 	debug_printf(DP_ERROR, "Invalid format string.\n");
+
 	return;
 
 ok:
@@ -71,29 +75,38 @@ ok:
 
 int main_show(int argc, char* argv[argc])
 {
+	const char* in_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &in_file, "input"),
+	};
+
 	bool meta = false;
 	int showdim = -1;
-	const char* sep = strdup("\t");
-	const char* fmt = strdup("%+.6e%+.6ei");
+	const char* sep = "\t";
+	const char* fmt = "%+.6e%+.6ei";
 
 	const struct opt_s opts[] = {
+
 		OPT_SET('m', &meta, "show meta data"),
 		OPT_INT('d', &showdim, "dim", "show size of dimension"),
 		OPT_STRING('s', &sep, "sep", "use <sep> as the separator"),
 		OPT_STRING('f', &fmt, "format", "use <format> as the format. Default: \"%%+.6e%%+.6ei\""),
 	};
 
-	cmdline(&argc, argv, 1, 1, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
-	unsigned int N = DIMS;
+	int N = DIMS;
 
 	long dims[N];
-	complex float* data = load_cfl(argv[1], N, dims);
+	complex float* data = load_cfl(in_file, N, dims);
 
 	if (-1 != showdim) {
 
 		assert((showdim >= 0) && (showdim < (int)N));
 		bart_printf("%ld\n", dims[showdim]);
+
 		goto out;
 	}
 
@@ -103,7 +116,7 @@ int main_show(int argc, char* argv[argc])
 		bart_printf("Dimensions: %d\n", N);	// FIXME always DIMS
 		bart_printf("AoD:");
 
-		for (unsigned int i = 0; i < N; i++)
+		for (int i = 0; i < N; i++)
 			bart_printf("\t%ld", dims[i]);
 
 		bart_printf("\n");
@@ -115,7 +128,5 @@ int main_show(int argc, char* argv[argc])
 
 out:
 	unmap_cfl(N, dims, data);
-	xfree(sep);
-	xfree(fmt);
 	return 0;
 }
