@@ -89,6 +89,9 @@ const struct ellipsis_s phantom_geo4[1] = {
 	{	0.8,   { .1,   .7   }, { .6,     0  },	   0},
 };
 
+const struct ellipsis_s phantom_geo5[1] = {
+	{      1.,   { .5,   -.5   }, { 0.,     0.  },	   0  },
+};
 
 const struct ellipsis_bs phantom_tubes[21] = {
 	{{	1.,	{ .125,	.125	},	{ -0.13, -0.19 },	0.,	}, false },
@@ -115,6 +118,63 @@ const struct ellipsis_bs phantom_tubes[21] = {
 };
 
 
+// NIST Phantom Geometry of T2 Sphere
+//	Stupic, KF, Ainslie, M, Boss, MA, et al.
+//	A standard system phantom for magnetic resonance imaging.
+//	Magn Reson Med. 2021; 86: 1194– 1211. https://doi.org/10.1002/mrm.28779
+const struct ellipsis_bs nist_phantom_t2[29] = {
+	/*01*/	{{	1.,	{ .074,	.074		},	{ 0., 0.254*2-0.074 },		0.,	}, false },
+	/*02*/	{{	1.,	{ .074,	.074		},	{ 0.096*2+0.074, 0.212*2-0.074 },	0.,	}, false },
+	/*03*/	{{	1.,	{ .074,	.074		},	{ 0.176*2+0.074, 0.104*2-0.074 },	0.,	}, false },
+	/*04*/	{{	1.,	{ .074,	.074		},	{ 0.176*2+0.074, -0.104*2+0.074 },	0.,	}, false },
+	/*05*/	{{	1.,	{ .074,	.074		},	{ 0.096*2+0.074, -0.212*2+0.074 },	0.,	}, false },
+	/*06*/	{{	1.,	{ .074,	.074		},	{ 0., -0.254*2+0.074 },	0.,	}, false },
+	/*07*/	{{	1.,	{ .074,	.074		},	{ -(0.096*2+0.074), -(0.212*2-0.074) },	0.,	}, false },
+	/*08*/	{{	1.,	{ .074,	.074		},	{ -(0.176*2+0.074), -(0.104*2-0.074) },	0.,	}, false },
+	/*09*/	{{	1.,	{ .074,	.074		},	{ -(0.176*2+0.074), 0.104*2-0.074 },	0.,	}, false },
+	/*10*/	{{	1.,	{ .074,	.074		},	{ -(0.096*2+0.074), 0.212*2-0.074 },	0.,	}, false },
+	/*11*/	{{	1.,	{ .074,	.074		},	{ -(0.053*2+0.074), 0.123*2-0.074 },	0.,	}, false },
+	/*12*/	{{	1.,	{ .074,	.074		},	{ 0.053*2+0.074, 0.123*2-0.074 },	0.,	}, false },
+	/*13*/	{{	1.,	{ .074,	.074		},	{ 0.053*2+0.074, -0.123*2+0.074 },	0.,	}, false },
+	/*14*/	{{	1.,	{ .074,	.074		},	{ -(0.053*2+0.074), -(0.123*2-0.074) },	0.,	}, false },
+	/*Back*/	{{	1.,	{ .81,	.81		},	{ 0.00,  0.00 },	0.,	}, true },
+	/*01*/	{{	-1.,	{ .088,	.088		},	{ 0., 0.254*2-0.074 },		0.,	}, true },
+	/*02*/	{{	-1.,	{ .088,	.088		},	{ 0.096*2+0.074, 0.212*2-0.074 },	0.,	}, true },
+	/*03*/	{{	-1.,	{ .088,	.088		},	{ 0.176*2+0.074, 0.104*2-0.074 },	0.,	}, true },
+	/*04*/	{{	-1.,	{ .088,	.088		},	{ 0.176*2+0.074, -0.104*2+0.074 },	0.,	}, true },
+	/*05*/	{{	-1.,	{ .088,	.088		},	{ 0.096*2+0.074, -0.212*2+0.074 },	0.,	}, true },
+	/*06*/	{{	-1.,	{ .088,	.088		},	{ 0., -0.254*2+0.074 },	0.,	}, true },
+	/*07*/	{{	-1.,	{ .088,	.088		},	{ -(0.096*2+0.074), -(0.212*2-0.074) },	0.,	}, true },
+	/*08*/	{{	-1.,	{ .088,	.088		},	{ -(0.176*2+0.074), -(0.104*2-0.074) },	0.,	}, true },
+	/*09*/	{{	-1.,	{ .088,	.088		},	{ -(0.176*2+0.074), 0.104*2-0.074 },	0.,	}, true },
+	/*10*/	{{	-1.,	{ .088,	.088		},	{ -(0.096*2+0.074), 0.212*2-0.074 },	0.,	}, true },
+	/*11*/	{{	-1.,	{ .088,	.088		},	{ -(0.053*2+0.074), 0.123*2-0.074 },	0.,	}, true },
+	/*12*/	{{	-1.,	{ .088,	.088		},	{ 0.053*2+0.074, 0.123*2-0.074 },	0.,	}, true },
+	/*13*/	{{	-1.,	{ .088,	.088		},	{ 0.053*2+0.074, -0.123*2+0.074 },	0.,	}, true },
+	/*14*/	{{	-1.,	{ .088,	.088		},	{ -(0.053*2+0.074), -(0.123*2-0.074) },	0.,	}, true },
+};
+
+
+// T1 T2 Phantom based on reference phantom by Diagnostic Sonar LTD (Scotland, UK)
+const struct ellipsis_bs phantom_sonar[15] = {
+	/*01*/	{{	1.,	{ .135,	.135		},	        { 0., 0.45 },		                        0.,	}, false },
+	/*02*/	{{	1.,	{ .135,	.135		},	{ 0.866*0.45, 0.5*0.45 },	        0.,	}, false },
+	/*03*/	{{	1.,	{ .135,	.135		},	{ 0.866*0.45, -0.5*0.45 },	0.,	}, false },
+	/*04*/	{{	1.,	{ .135,	.135		},	        { 0., -0.45 },	                                0.,	}, false },
+	/*05*/	{{	1.,	{ .135,	.135		},	{ -0.866*0.45, -0.5*0.45 },	0.,	}, false },
+	/*06*/	{{	1.,	{ .135,	.135		},	{ -0.866*0.45, 0.5*0.45 },	0.,	}, false },
+	/*07 (center)*/	{{	1.,	{ .16,	.16		},	{ 0.00,  0.00 },	                        0.,	}, false },
+	/*Back*/	{{	1.,	{ .75,	.75		},	{ 0.00,  0.00 },	                        0.,	}, true },
+	/*01*/	{{	-1.,	{ .15,	.15		},	        { 0., 0.45 },		                        0.,	}, true },
+	/*02*/	{{	-1.,	{ .15,	.15		},	{ 0.866*0.45, 0.5*0.45 },	        0.,	}, true },
+	/*03*/	{{	-1.,	{ .15,	.15		},	{ 0.866*0.45, -0.5*0.45 },	0.,	}, true },
+	/*04*/	{{	-1.,	{ .15,	.15		},	        { 0., -0.45 },	                                0.,	}, true },
+	/*05*/	{{	-1.,	{ .15,	.15		},	{ -0.866*0.45, -0.5*0.45 },	0.,	}, true },
+	/*06*/	{{	-1.,	{ .15,	.15		},	{ -0.866*0.45, 0.5*0.45 },	0.,	}, true },
+	/*07 (center)*/	{{	-1.,	{ .175,	.175		},	{ 0.00,  0.00 },	                        0.,	}, true },
+};
+
+
 /* Magnetic Resonance in Medicine 58:430--436 (2007)
  * Three-Dimensional Analytical Magnetic Resonance
  * Imaging Phantom in the Fourier Domain
