@@ -28,12 +28,18 @@
 
 #include "noncart/traj.h"
 
-static const char usage_str[] = "<output>";
 static const char help_str[] = "Computes k-space trajectories.";
 
 
 int main_traj(int argc, char* argv[argc])
 {
+	const char* out_file= NULL;
+
+	struct arg_s args[] = {
+
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
 	int X = 128;
 	int Y = 128;
 	int D = -1;
@@ -41,6 +47,7 @@ int main_traj(int argc, char* argv[argc])
 	int mb = 1;
 	int turns = 1;
 	float rot = 0.;
+	float over = 1.;
 
 	struct traj_conf conf = traj_defaults;
 
@@ -71,6 +78,7 @@ int main_traj(int argc, char* argv[argc])
 		OPT_SET('H', &conf.half_circle_gold, "halfCircle golden-ratio sampling"),
 		OPT_INT('s', &conf.tiny_gold, "# Tiny GA", "tiny golden angle"),
 		OPT_SET('D', &conf.full_circle, "projection angle in [0,360°), else in [0,180°)"),
+		OPT_FLOAT('o', &over, "o", "oversampling factor"),
 		OPT_FLOAT('R', &rot, "phi", "rotate"),
 		OPT_FLVEC3('q', &gdelays[0], "delays", "gradient delays: x, y, xy"),
 		OPT_FLVEC3('Q', &gdelays[1], "delays", "(gradient delays: z, xz, yz)"),
@@ -79,14 +87,15 @@ int main_traj(int argc, char* argv[argc])
 		OPT_SET('c', &conf.asym_traj, "asymmetric trajectory [DC sampled]"),
 		OPT_SET('E', &conf.mems_traj, "multi-echo multi-spoke trajectory"),
 		OPT_VEC2('z', &z_usamp, "Ref:Acel", "Undersampling in z-direction."),
-		OPT_STRING('C', &custom_angle_file, "file", "custom_angle file [phi + i * psi]"),
-		OPT_STRING('V', &gdelays_file, "file", "(custom_gdelays)"),
+		OPT_INFILE('C', &custom_angle_file, "file", "custom_angle file [phi + i * psi]"),
+		OPT_INFILE('V', &gdelays_file, "file", "(custom_gdelays)"),
 	};
 
-	cmdline(&argc, argv, 1, 1, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
+
 	// Load custom_angle
 	long sdims[DIMS];
 	complex float* custom_angle_vals = NULL;
@@ -106,6 +115,11 @@ int main_traj(int argc, char* argv[argc])
 		Y = sdims[0];
 	}
 
+	if (over <= 0.)
+		error("Oversampling factor must be positive.\n");
+
+	X *= over;
+
 	int tot_sp = Y * E * mb * turns;	// total number of lines/spokes
 	int N = X * tot_sp / conf.accel;
 
@@ -198,7 +212,7 @@ int main_traj(int argc, char* argv[argc])
 	}
 
 
-	complex float* samples = create_cfl(argv[1], DIMS, dims);
+	complex float* samples = create_cfl(out_file, DIMS, dims);
 
 	md_clear(DIMS, dims, samples, CFL_SIZE);
 
@@ -210,6 +224,7 @@ int main_traj(int argc, char* argv[argc])
 
 	int p = 0;
 	long pos[DIMS] = { 0 };
+	double phin1 = 0;
 
 	do {
 		int i = pos[PHS1_DIM];
@@ -219,8 +234,6 @@ int main_traj(int argc, char* argv[argc])
 
 		if (conf.radial) {
 
-			int s = j;
-
 			/* Calculate read-out samples
 			 * for symmetric trajectory [DC between between sample no. X/2-1 and X/2, zero-based indexing]
 			 * or asymmetric trajectory [DC component at sample no. X/2, zero-based indexing]
@@ -238,7 +251,7 @@ int main_traj(int argc, char* argv[argc])
 
 			double angle = 0.;
 
-			for (unsigned int d = 1; d < DIMS; d++)
+			for (int d = 1; d < DIMS; d++)
 				angle += pos[d] * base_angle[d];
 
 
@@ -252,8 +265,28 @@ int main_traj(int argc, char* argv[argc])
 
 			if (conf.d3d) {
 
-				int split = sqrtf(Y);
-				angle2 = s * M_PI / Y * (conf.full_circle ? 2 : 1) * split;
+				/* Saff EB., Kuijlaars ABJ.
+				 * Distributing many points on a sphere.
+				 * The Mathematical Intelligencer 1997;19:11.
+				 * DOI:10.1007/BF03024331
+				 */
+
+				int Y2 = Y;
+
+				if (!conf.full_circle) // half sphere
+					Y2 = Y * 2;
+
+				double hn = -1.0 + (double)(2 * j) / (Y2 - 1);
+
+				if ((j + 1 == Y) || (j == 0))
+					angle = 0;
+				else
+					angle = fmod(phin1 + 3.6 / sqrt(Y2 * (1.0 - hn * hn)), 2. * M_PI);
+
+				if (i + 1 == X)	// FIXME: a non-recursive formula?
+					phin1 = angle;
+
+				angle2 = acos(hn) - M_PI / 2.;
 
 				if (NULL != custom_angle_vals)
 					angle2 = cimagf(custom_angle_vals[j]);
@@ -290,10 +323,10 @@ int main_traj(int argc, char* argv[argc])
 
 				float delay = 0.;
 
-				for (unsigned int i = 0; i < 3; i++)
+				for (int i = 0; i < 3; i++)
 					delay += read_dir[i] * d[i];
 
-				for (unsigned int i = 0; i < 3; i++)
+				for (int i = 0; i < 3; i++)
 					d[i] = delay * read_dir[i];
 			}
 
@@ -312,17 +345,19 @@ int main_traj(int argc, char* argv[argc])
 
 	} while(md_next(DIMS, dims, ~1L, pos));
 
+	md_zsmul(DIMS, dims, samples, samples, 1. / over);
+
 	assert(p == N - 0);
 
 	if (NULL != gdelays2)
 		unmap_cfl(DIMS, gdims, gdelays2);
 
 	if (NULL != custom_angle_vals)
-		unmap_cfl(3, sdims, custom_angle_vals);
+		unmap_cfl(DIMS, sdims, custom_angle_vals);
 
-	unmap_cfl(3, dims, samples);
+	unmap_cfl(DIMS, dims, samples);
 
-	exit(0);
+	return 0;
 }
 
 
