@@ -21,6 +21,9 @@
 #define UT_ASSERT(test)	\
 	return ((test) || (debug_printf(DP_ERROR, "%s:%d assertion `%s` failed.\n", __func__, __LINE__, #test), false))
 
+#define UT_ASSERT2(test)	\
+	if (!(test)) return (debug_printf(DP_ERROR, "%s:%d assertion `%s` failed.\n", __func__, __LINE__, #test), false)
+
 #define UT_TOL 1E-6
 
 
